@@ -256,39 +256,6 @@ use_netavark() {
     # N/B: The CNI packages are still installed and available. This is
     # on purpose, since CI needs to verify the selection mechanisms are
     # functional when both are available.
-
-    # See ./contrib/cirrus/CIModes.md.
-    # Vars defined by cirrus-ci
-    # shellcheck disable=SC2154
-    if [[ ! "$OS_RELEASE_ID" =~ "debian" ]] && \
-       [[ "$CIRRUS_CHANGE_TITLE" =~ CI:[AN]V[AN]V= ]]
-    then
-        # shellcheck disable=SC2154
-        if [[ "$CIRRUS_PR_DRAFT" != "true" ]]; then
-            die "Magic 'CI:NVAV=*' string can only be used on DRAFT PRs"
-        fi
-
-        magickind=$(sed -r -e 's~(.*CI:[AN]V[AN]V=)(\w+)(.*)~\2~' <<<"$CIRRUS_CHANGE_TITLE")
-
-        # The update source scheme is defined during VM image build.
-        # See c/automation_images repo. cache_images/fedora_packaging.sh
-        repokind="updates-testing"  # $DISTRO_NV==$FEDORA_NAME
-        # shellcheck disable=SC2154
-        if [[ "$DISTRO_NV" =~ $PRIOR_FEDORA_NAME ]]; then
-            repokind="updates"
-        # else we're not running fedora, or .cirrus.yml env. vars are setup wrong.
-        fi
-
-        if [[ "$magickind" == "update" ]]; then
-            warn "Updating netavark/aardvark RPM packages from ***the fedora $repokind repo.***"
-        elif [[ "$magickind" == "main" ]]; then
-            warn "Installing latest netavark/aardvark packages from their main branches using ***the podman-next COPR repo***"
-            showrun dnf copr enable rhcontainerbot/podman-next -y
-        else
-            die "Unknown CI:NVAV= '$magickind' keyword.  Only 'update' and 'main' are supported."
-        fi
-        showrun dnf upgrade -y netavark aardvark-dns
-    fi
 }
 
 # Remove all files provided by the distro version of podman.
@@ -335,37 +302,3 @@ remove_packaged_podman_files() {
     # Be super extra sure and careful vs performant and completely safe
     sync && echo 3 > /proc/sys/vm/drop_caches || true
 }
-
-# Execute make localbenchmarks in $CIRRUS_WORKING_DIR/data
-# for preserving as a task artifact.
-localbenchmarks() {
-    local datadir envnames envname
-    req_env_vars DISTRO_NV PODBIN_NAME PRIV_NAME TEST_ENVIRON TEST_FLAVOR
-    req_env_vars VM_IMAGE_NAME EC2_INST_TYPE
-
-    datadir=$CIRRUS_WORKING_DIR/data
-    mkdir -p $datadir
-
-    envnames=$(passthrough_envars | sort);
-    (
-      echo "# Env. var basis for benchmarks benchmarks."
-      for envname in $envnames; do
-        printf "$envname=%q\n" "${!envname}"
-      done
-
-      echo "# Machine details for data-comparison sake, not actual env. vars."
-      # Checked above in req_env_vars
-      # shellcheck disable=SC2154
-      echo "\
-BENCH_ENV_VER=1
-CPUTOTAL=$(grep -ce '^processor' /proc/cpuinfo)
-INST_TYPE=$EC2_INST_TYPE
-MEMTOTALKB=$(awk -F: '$1 == "MemTotal" { print $2 }' </proc/meminfo | sed -e "s/^ *//" | cut -d ' ' -f 1)
-UNAME_R=$(uname -r)
-UNAME_M=$(uname -m)
-"
-    ) > $datadir/benchmarks.env
-    make localbenchmarks | tee $datadir/benchmarks.raw
-    msg "Processing raw benchmarks output"
-    hack/parse-localbenchmarks < $datadir/benchmarks.raw | tee $datadir/benchmarks.csv
-}
