@@ -6,34 +6,14 @@ import (
 	"os/exec"
 	"os/user"
 	"path/filepath"
+	"strconv"
 
-	. "github.com/containers/podman/v4/test/utils"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman Info", func() {
-	var (
-		tempdir    string
-		err        error
-		podmanTest *PodmanTestIntegration
-	)
-
-	BeforeEach(func() {
-		tempdir, err = CreateTempDirInTempDir()
-		if err != nil {
-			os.Exit(1)
-		}
-		podmanTest = PodmanTestCreate(tempdir)
-		podmanTest.Setup()
-	})
-
-	AfterEach(func() {
-		podmanTest.Cleanup()
-		f := CurrentGinkgoTestDescription()
-		processTestResult(f)
-	})
 
 	It("podman info --format json", func() {
 		tests := []struct {
@@ -180,6 +160,11 @@ var _ = Describe("Podman Info", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).To(Exit(0))
 		Expect(session.OutputToString()).To(Equal(want))
+
+		session = podmanTest.Podman([]string{"info", "--format", "{{.Host.NetworkBackendInfo.Backend}}"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).To(Exit(0))
+		Expect(session.OutputToString()).To(Equal(want))
 	})
 
 	It("Podman info: check desired database backend", func() {
@@ -196,4 +181,30 @@ var _ = Describe("Podman Info", func() {
 		Expect(session).To(Exit(0))
 		Expect(session.OutputToString()).To(Equal(want))
 	})
+
+	It("Podman info: check lock count", Serial, func() {
+		// This should not run on architectures and OSes that use the file locks backend.
+		// Which, for now, is Linux + RISCV and FreeBSD, neither of which are in CI - so
+		// no skips.
+		info1 := podmanTest.Podman([]string{"info", "--format", "{{ .Host.FreeLocks }}"})
+		info1.WaitWithDefaultTimeout()
+		Expect(info1).To(Exit(0))
+		free1, err := strconv.Atoi(info1.OutputToString())
+		Expect(err).To(Not(HaveOccurred()))
+
+		ctr := podmanTest.Podman([]string{"create", ALPINE, "top"})
+		ctr.WaitWithDefaultTimeout()
+		Expect(ctr).To(Exit(0))
+
+		info2 := podmanTest.Podman([]string{"info", "--format", "{{ .Host.FreeLocks }}"})
+		info2.WaitWithDefaultTimeout()
+		Expect(info2).To(Exit(0))
+		free2, err := strconv.Atoi(info2.OutputToString())
+		Expect(err).To(Not(HaveOccurred()))
+
+		// Effectively, we are checking that 1 lock has been taken.
+		// We do this by comparing the number of locks after (plus 1), to the number of locks before.
+		// Don't check absolute numbers because there is a decent chance of contamination, containers that were never removed properly, etc.
+		Expect(free1).To(Equal(free2 + 1))
+	})
 })
