@@ -7,33 +7,12 @@ import (
 	"runtime"
 
 	. "github.com/containers/podman/v4/test/utils"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman pull", func() {
-	var (
-		tempdir    string
-		err        error
-		podmanTest *PodmanTestIntegration
-	)
-
-	BeforeEach(func() {
-		tempdir, err = CreateTempDirInTempDir()
-		if err != nil {
-			os.Exit(1)
-		}
-		podmanTest = PodmanTestCreate(tempdir)
-		podmanTest.Setup()
-	})
-
-	AfterEach(func() {
-		podmanTest.Cleanup()
-		f := CurrentGinkgoTestDescription()
-		processTestResult(f)
-
-	})
 
 	It("podman pull multiple images with/without tag/digest", func() {
 		session := podmanTest.Podman([]string{"pull", "busybox:musl", "alpine", "alpine:latest", "quay.io/libpod/cirros", "quay.io/libpod/testdigest_v2s2@sha256:755f4d90b3716e2bf57060d249e2cd61c9ac089b1233465c5c2cb2d7ee550fdb"})
@@ -44,8 +23,7 @@ var _ = Describe("Podman pull", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(125))
 		expectedError := "initializing source docker://ibetthisdoesnotexistfr:random"
-		found, _ := session.ErrorGrepString(expectedError)
-		Expect(found).To(Equal(true))
+		Expect(session.ErrorToString()).To(ContainSubstring(expectedError))
 
 		session = podmanTest.Podman([]string{"rmi", "busybox:musl", "alpine", "quay.io/libpod/cirros", "testdigest_v2s2@sha256:755f4d90b3716e2bf57060d249e2cd61c9ac089b1233465c5c2cb2d7ee550fdb"})
 		session.WaitWithDefaultTimeout()
@@ -84,6 +62,73 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 	})
 
+	It("podman pull and run on split imagestore", func() {
+		SkipIfRemote("podman-remote does not support setting external imagestore")
+		imgName := "splitstoretest"
+
+		// Make alpine write-able
+		session := podmanTest.Podman([]string{"build", "--pull=never", "--tag", imgName, "build/basicalpine"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		tmpDir := filepath.Join(podmanTest.TempDir, "splitstore")
+		outfile := filepath.Join(podmanTest.TempDir, "image.tar")
+
+		save := podmanTest.Podman([]string{"save", "-o", outfile, "--format", "oci-archive", imgName})
+		save.WaitWithDefaultTimeout()
+		Expect(save).Should(Exit(0))
+
+		rmi := podmanTest.Podman([]string{"rmi", imgName})
+		rmi.WaitWithDefaultTimeout()
+		Expect(rmi).Should(Exit(0))
+
+		// load to splitstore
+		result := podmanTest.Podman([]string{"load", "--imagestore", tmpDir, "-q", "-i", outfile})
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(Exit(0))
+
+		// tag busybox to busybox-test in graphroot since we can delete readonly busybox
+		session = podmanTest.Podman([]string{"tag", "quay.io/libpod/busybox:latest", "busybox-test"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		session = podmanTest.Podman([]string{"images", "--imagestore", tmpDir})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).To(ContainSubstring(imgName))
+		Expect(session.OutputToString()).To(ContainSubstring("busybox-test"))
+
+		// Test deleting image in graphroot even when `--imagestore` is set
+		session = podmanTest.Podman([]string{"rmi", "--imagestore", tmpDir, "busybox-test"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+
+		// Images without --imagestore should not contain alpine
+		session = podmanTest.Podman([]string{"images"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).To(Not(ContainSubstring(imgName)))
+
+		// Set `imagestore` in `storage.conf` and container should run.
+		configPath := filepath.Join(podmanTest.TempDir, ".config", "containers", "storage.conf")
+		os.Setenv("CONTAINERS_STORAGE_CONF", configPath)
+		defer func() {
+			os.Unsetenv("CONTAINERS_STORAGE_CONF")
+		}()
+
+		err = os.MkdirAll(filepath.Dir(configPath), os.ModePerm)
+		Expect(err).ToNot(HaveOccurred())
+		storageConf := []byte(fmt.Sprintf("[storage]\nimagestore=\"%s\"", tmpDir))
+		err = os.WriteFile(configPath, storageConf, os.ModePerm)
+		Expect(err).ToNot(HaveOccurred())
+
+		session = podmanTest.Podman([]string{"run", "--name", "test", "--rm",
+			imgName, "echo", "helloworld"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).To(ContainSubstring("helloworld"))
+	})
+
 	It("podman pull by digest", func() {
 		session := podmanTest.Podman([]string{"pull", "quay.io/libpod/testdigest_v2s2@sha256:755f4d90b3716e2bf57060d249e2cd61c9ac089b1233465c5c2cb2d7ee550fdb"})
 		session.WaitWithDefaultTimeout()
@@ -288,8 +333,7 @@ var _ = Describe("Podman pull", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(125))
 		expectedError := "Unexpected tar manifest.json: expected 1 item, got 2"
-		found, _ := session.ErrorGrepString(expectedError)
-		Expect(found).To(Equal(true))
+		Expect(session.ErrorToString()).To(ContainSubstring(expectedError))
 
 		// Now pull _one_ image from a multi-image archive via the name
 		// and index syntax.
@@ -314,15 +358,13 @@ var _ = Describe("Podman pull", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(125))
 		expectedError = "Tag \"foo.com/does/not/exist:latest\" not found"
-		found, _ = session.ErrorGrepString(expectedError)
-		Expect(found).To(Equal(true))
+		Expect(session.ErrorToString()).To(ContainSubstring(expectedError))
 
 		session = podmanTest.Podman([]string{"pull", "docker-archive:./testdata/docker-two-images.tar.xz:@2"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(125))
 		expectedError = "Invalid source index @2, only 2 manifest items available"
-		found, _ = session.ErrorGrepString(expectedError)
-		Expect(found).To(Equal(true))
+		Expect(session.ErrorToString()).To(ContainSubstring(expectedError))
 	})
 
 	It("podman pull from oci-archive", func() {
@@ -559,4 +601,89 @@ var _ = Describe("Podman pull", func() {
 		Expect(session).Should(Exit(0))
 		Expect(session.ErrorToString()).To(BeEmpty())
 	})
+
+	Describe("podman pull and decrypt", func() {
+
+		decryptionTestHelper := func(imgPath string) *PodmanSessionIntegration {
+			bitSize := 1024
+			keyFileName := filepath.Join(podmanTest.TempDir, "key")
+			publicKeyFileName, privateKeyFileName, err := WriteRSAKeyPair(keyFileName, bitSize)
+			Expect(err).ToNot(HaveOccurred())
+
+			wrongKeyFileName := filepath.Join(podmanTest.TempDir, "wrong_key")
+			_, wrongPrivateKeyFileName, err := WriteRSAKeyPair(wrongKeyFileName, bitSize)
+			Expect(err).ToNot(HaveOccurred())
+
+			session := podmanTest.Podman([]string{"push", "--encryption-key", "jwe:" + publicKeyFileName, "--tls-verify=false", "--remove-signatures", ALPINE, imgPath})
+			session.WaitWithDefaultTimeout()
+
+			session = podmanTest.Podman([]string{"rmi", ALPINE})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(0))
+
+			// Pulling encrypted image without key should fail
+			session = podmanTest.Podman([]string{"pull", imgPath})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(125))
+
+			// Pulling encrypted image with wrong key should fail
+			session = podmanTest.Podman([]string{"pull", "--decryption-key", wrongPrivateKeyFileName, imgPath})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(125))
+
+			// Pulling encrypted image with correct key should pass
+			session = podmanTest.Podman([]string{"pull", "--decryption-key", privateKeyFileName, imgPath})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(0))
+			session = podmanTest.Podman([]string{"images"})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(0))
+
+			return session
+		}
+
+		It("From oci", func() {
+			SkipIfRemote("Remote pull neither supports oci transport, nor decryption")
+
+			podmanTest.AddImageToRWStore(ALPINE)
+
+			bbdir := filepath.Join(podmanTest.TempDir, "busybox-oci")
+			imgPath := fmt.Sprintf("oci:%s", bbdir)
+
+			session := decryptionTestHelper(imgPath)
+
+			Expect(session.LineInOutputContainsTag(filepath.Join("localhost", bbdir), "latest")).To(BeTrue())
+		})
+
+		It("From local registry", func() {
+			SkipIfRemote("Remote pull does not support decryption")
+
+			if podmanTest.Host.Arch == "ppc64le" {
+				Skip("No registry image for ppc64le")
+			}
+
+			podmanTest.AddImageToRWStore(ALPINE)
+
+			if isRootless() {
+				err := podmanTest.RestoreArtifact(REGISTRY_IMAGE)
+				Expect(err).ToNot(HaveOccurred())
+			}
+			lock := GetPortLock("5000")
+			defer lock.Unlock()
+			session := podmanTest.Podman([]string{"run", "-d", "--name", "registry", "-p", "5000:5000", REGISTRY_IMAGE, "/entrypoint.sh", "/etc/docker/registry/config.yml"})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(Exit(0))
+
+			if !WaitContainerReady(podmanTest, "registry", "listening on", 20, 1) {
+				Skip("Cannot start docker registry.")
+			}
+
+			imgPath := "localhost:5000/my-alpine"
+
+			session = decryptionTestHelper(imgPath)
+
+			Expect(session.LineInOutputContainsTag(imgPath, "latest")).To(BeTrue())
+		})
+	})
+
 })
